import enum
import os

from youqu3._setting._dynamic import _DynamicSetting


class _Setting(_DynamicSetting):
    """YouQu Config"""

    PASSWORD: str = os.environ.get("YOUQU_PASSWORD") or "1"

    TIMEOUT = os.environ.get("YOUQU_TIMEOUT") or 300
    LOG_LEVEL = os.environ.get("YOUQU_LOG_LEVEL") or "INFO"
    RERUNS = os.environ.get("YOUQU_RERUNS") or 1
    RECORD_FAILED_NUM = os.environ.get("YOUQU_RECORD_FAILED_NUM") or 1
    if os.popen("pip3 list | grep pytest-record-video").read().strip() == "":
        RECORD_FAILED_NUM = None

    # REPORT SERVER
    REPORT_SERVER_IP = "127.0.0.1/127.0.0.2"
    REPORT_PORT = 5656
    REPORT_BASE_PATH = "~/report"

    # SLAVES
    SLAVES = os.environ.get("SLAVES")

    PYPI_MIRROR = "https://pypi.tuna.tsinghua.edu.cn/simple"

    @enum.unique
    class FixedCsvTitle(enum.Enum):
        case_id = "脚本ID"
        skip_reason = "跳过原因"
        fixed = "确认修复"
        removed = "废弃用例"

    @enum.unique
    class ConfStr(enum.Enum):
        SKIP = "skip"
        SKIPIF = "skipif"
        FIXED = "fixed"
        PASSED = "passed"
        FAILED = "failed"
        SKIPPED = "skipped"
        REMOVED = "removed"
        SKIP_INDEX = "skip_index"
        FIXED_INDEX = "fixed_index"
        REMOVED_INDEX = "removed_index"
        PMS_ID_INDEX = "pms_id_index"


setting = _Setting()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import pathlib
import re
import socket
from concurrent.futures import ALL_COMPLETED
from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import wait
from copy import deepcopy
from time import sleep

from funnylog2 import logger
from nocmd import Cmd, RemoteCmd

from youqu3 import setting


class Remote:
    __author__ = "mikigo <huangmingqiang@uniontech.com>"

    def __init__(
            self,
            clients=None,
            workdir=None,
            path=None,
            keywords=None,
            tags=None,
            slaves=None,
            txt=None,
            reruns=None,
            job_start=None,
            job_end=None,
            pytest_opt=None,
            record_failed_num=None,
            **kwargs
    ):
        logger("INFO")

        self.workdir = workdir
        self.path = path
        self.keywords = keywords
        self.tags = tags
        self.clients = clients
        self.slaves = slaves
        self.txt = txt
        self.reruns = reruns
        self.job_start = job_start
        self.job_end = job_end
        self.pytest_opt = pytest_opt
        self.record_failed_num = record_failed_num

        if not self.clients:
            raise ValueError("REMOTE驱动模式, 未传入远程客户端信息：-c/--clients user@ip:pwd")
        self.group_type = False
        if "{" in self.clients and "}" in self.clients:
            self.group_type = True

        if self.group_type is False:
            self.cli_clients = {}
            _cli_clients = self.clients.split("/")
            for index, _client in enumerate(_cli_clients):
                _cli_client_info = re.findall(r"^(.+?)@(\d+\.\d+\.\d+\.\d+):{0,1}(.*?)$", _client)
                if _cli_client_info:
                    _c = list(_cli_client_info[0])
                    if _c[2] == "":
                        _c[2] = setting.PASSWORD
                    connected = self.check_remote_connected(*_c)
                    self.cli_clients[f"client{index + 1}{f'' if connected else '-X'}"] = _c
        else:
            self.cli_groups = {}
            groups = re.findall(r'\{(.*?)\}', self.clients)
            for group_index, group in enumerate(groups):
                cli_clients = {}
                for client_index, _client in enumerate(group.split("/")):
                    _cli_client_info = re.findall(r"^(.+?)@(\d+\.\d+\.\d+\.\d+):{0,1}(.*?)$", _client)
                    if _cli_client_info:
                        _c = list(_cli_client_info[0])
                        if _c[2] == "":
                            _c[2] = setting.PASSWORD
                        connected = self.check_remote_connected(*_c)
                        cli_clients[f"client{client_index + 1}{f'' if connected else '-X'}"] = _c
                self.cli_groups[f"group{group_index + 1}"] = cli_clients

        self.server_rootdir = pathlib.Path("driver").absolute()
        self.rootdir_name = self.server_rootdir.name
        self.client_rootdir = lambda x: f"/home/{x}/{self.rootdir_name}_{setting.TIME_STRING}"
        self.client_report_path = lambda x: f"{self.client_rootdir(x)}/report"
        self.client_html_report_path = lambda x: f"{self.client_report_path(x)}/html"
        self.client_json_report_path = lambda x: f"{self.client_report_path(x)}/json"

        self.rsync = 'rsync -av -e "ssh -o StrictHostKeyChecking=no"'
        self.empty = "> /dev/null 2>&1"

        self.collection_json = False
        self.server_json_dir_id = None
        self.inside_filepath = None

        from funnylog2.config import config as funnylog2_config
        funnylog2_config.LOG_FILE_PATH = self.server_rootdir

    def check_remote_connected(self, user, _ip, password):
        logger.info(f"Checking remote: {user, _ip, password}")
        try:
            return_code = RemoteCmd(user, _ip, password).remote_run("hostname -I", use_sshpass=True)
            if return_code == 0:
                return True
            return False
        except socket.timeout:
            return False

    def send_code(self, user, _ip, password):
        logger.info(f"开始发送代码到测试机 - < {user}@{_ip} >")
        logger.info(self.client_rootdir(user))
        RemoteCmd(user, _ip, password).remote_sudo_run(f"rm -rf {self.client_rootdir(user)}")
        RemoteCmd(user, _ip, password).remote_run(f"mkdir -p {self.client_rootdir(user)}")
        exclude = ""
        for i in [
            "__pycache__",
            ".pytest_cache",
            ".vscode",
            ".idea",
            ".git",
            ".github",
            ".venv",
            "report",
            ".gitignore",
            "LICENSE",
            "Pipfile",
            "Pipfile.lock",
            "README.md",
        ]:
            exclude += f"--exclude='{i}' "
        _, return_code = Cmd.expect_run(
            f"/bin/bash -c '{self.rsync} {exclude} {self.server_rootdir}/* {user}@{_ip}:{self.client_rootdir(user)}/'",
            events={'password': f'{password}\n'},
            return_code=True,
            timeout=6000,
        )
        _, return_code = Cmd.expect_run(
            f"/bin/bash -c '{self.rsync} {self.server_rootdir}/.env {user}@{_ip}:{self.client_rootdir(user)}/'",
            events={'password': f'{password}\n'},
            return_code=True,
            timeout=600,
        )
        logger.info(f"代码发送{'成功' if return_code == 0 else '失败'} - < {user}@{_ip} >")

    def install_client_env(self, user, _ip, password):
        logger.info(f"开始安装环境 - < {user}@{_ip} >")
        remote = RemoteCmd(user, _ip, password)
        stdout, return_code = remote.remote_run("pip3 --version", return_code=True)
        if return_code != 0:
            remote.remote_sudo_run('apt update', use_sshpass=True)
            remote.remote_sudo_run('apt install python3-pip -y', use_sshpass=True)
            remote.remote_run(f'pip3 install -U pip -i {setting.PYPI_MIRROR} --break-system-packages', use_sshpass=True)
        remote.remote_run(f"pip3 install -U youqu3 -i {setting.PYPI_MIRROR} --break-system-packages", use_sshpass=True)
        return_code = remote.remote_run(
            f"export PATH=\$PATH:\$HOME/.local/bin; cd {self.client_rootdir(user)} && youqu3 envx",
            use_sshpass=True,
        )
        if return_code != 0:
            logger.info(f"环境安装失败 - < {user}@{_ip} >")
            raise EnvironmentError
        logger.info(f"环境安装成功 - < {user}@{_ip} >")

    def send_code_and_env(self, user, _ip, password):
        self.send_code(user, _ip, password)
        self.install_client_env(user, _ip, password)

    @staticmethod
    def makedirs(dirs):
        pathlib.Path(dirs).mkdir(parents=True, exist_ok=True)

    def get_back_report(self, user, _ip, password):
        server_html_path = f"{self.server_rootdir}/report/remote/{setting.TIME_STRING}_{_ip}_{self.rootdir_name}"
        self.makedirs(server_html_path)
        Cmd.expect_run(
            f"/bin/bash -c '{self.rsync} {user}@{_ip}:{self.client_report_path(user)}/* {server_html_path}/'",
            events={'password': f'{password}\n'},
            return_code=True
        )

    def get_back_all_report(self, client_list, clients):

        if len(clients) >= 2:
            _ps = []
            executor = ThreadPoolExecutor()
            for client in client_list[:-1]:
                user, _ip, password = clients.get(client)
                _p4 = executor.submit(self.get_back_report, user, _ip, password)
                _ps.append(_p4)
                sleep(2)
            user, _ip, password = clients.get(client_list[-1])
            self.get_back_report(user, _ip, password)
            wait(_ps, return_when=ALL_COMPLETED)
        else:
            user, _ip, password = clients.get(client_list[0])
            self.get_back_report(user, _ip, password)

    def read_tags_txt(self):
        youqu_tags_file = os.path.join(self.server_rootdir, "youqu-tags.txt")
        if os.path.exists(youqu_tags_file):
            with open(youqu_tags_file, "r", encoding="utf-8") as f:
                tags_txt = f.readlines()[0]
                return tags_txt
        return None

    def read_keywords_txt(self):
        youqu_keywords_file = os.path.join(self.server_rootdir, "youqu-keywords.txt")
        if os.path.exists(youqu_keywords_file):
            with open(youqu_keywords_file, "r", encoding="utf-8") as f:
                keywords_txt = f.readlines()[0]
                return keywords_txt
        return None

    def changedir_remote_cmd(self, user):
        return [
            "export PATH=\$PATH:\$HOME/.local/bin;",
            "cd",
            f"{self.client_rootdir(user)}/",
            "&&",
        ]

    @property
    def generate_cmd(self):
        cmd = ["youqu3-cargo", "run"]

        if self.workdir:
            if os.path.exists(self.workdir):
                cmd.extend(["--workdir", self.workdir])
            else:
                raise FileNotFoundError(f"workdir not found: {self.workdir}")
        if self.path:
            cmd.append(f"'{self.path}'")
        if self.inside_filepath:
            cmd.append(f"'{self.inside_filepath}'")

        keywords_txt = self.read_keywords_txt()
        if self.keywords:
            cmd.extend(["-k", f"'{self.keywords}'"])
        elif self.txt and keywords_txt is not None:
            cmd.extend(["-k", f"'{keywords_txt}'"])

        tags_txt = self.read_tags_txt()
        if self.tags:
            cmd.extend(["-t", f"'{self.tags}'"])
        elif self.txt and tags_txt is not None:
            cmd.extend(["-t", f"'{tags_txt}'"])
        if self.slaves:
            cmd.extend(["--slaves", f"'{self.slaves}'"])
        if self.reruns:
            cmd.extend(["--reruns", self.reruns or setting.RERUNS])
        if self.job_start:
            cmd.extend(["--job-start", f"'{self.job_start}'"])
        if self.job_end:
            cmd.extend(["--job-end", f"'{self.job_end}"])
        if self.pytest_opt:
            cmd.extend([i.strip() for i in self.pytest_opt])
        if self.record_failed_num or setting.RECORD_FAILED_NUM:
            cmd.extend(["--record-failed-num", f"{self.record_failed_num or setting.RECORD_FAILED_NUM}"])
        return cmd

    def run_test(self, user, _ip, password, use_sshpass=False):
        RemoteCmd(user, _ip, password).remote_run(
            " ".join(
                self.changedir_remote_cmd(user) + self.generate_cmd
            ),
            use_sshpass=use_sshpass,
        )

    @property
    def collection_only_cmd(self):
        return self.generate_cmd + ["--setup-plan"]

    @property
    def get_collection_only_cases(self):
        stdout = Cmd.run(f"cd {self.server_rootdir} && {' '.join(self.collection_only_cmd)}", timeout=600)
        lines = stdout.split("\n")
        _collection_cases = []
        for line in lines:
            line = line.strip()
            if line and " " not in line:
                _collection_cases.append(line.split("::")[0])
        collection_cases = set(_collection_cases)
        return collection_cases

    def parallel_run(self, client_name_list, clients):
        _ps = []
        executor = ThreadPoolExecutor()
        all_clients_name_list = deepcopy(list(clients.keys()))
        for i in all_clients_name_list:
            if i not in client_name_list:
                clients.pop(i)
        for client in list(clients.keys())[:-1]:
            user, _ip, password = clients.get(client)
            _p3 = executor.submit(self.run_test, user, _ip, password, True)
            _ps.append(_p3)
            sleep(1)
        user, _ip, password = list(clients.values())[-1]
        self.run_test(user, _ip, password, use_sshpass=True)
        wait(_ps, return_when=ALL_COMPLETED)
        sleep(5)

    def mul_do(self, func_obj, client_list, clients):
        if len(client_list) >= 2:
            executor = ThreadPoolExecutor()
            _ps = []
            for client in client_list[:-1]:
                user, _ip, password = clients.get(client)
                _p1 = executor.submit(func_obj, user, _ip, password)
                _ps.append(_p1)
            user, _ip, password = clients.get(client_list[-1])
            func_obj(user, _ip, password)
            wait(_ps, return_when=ALL_COMPLETED)
        else:
            user, _ip, password = clients.get(client_list[0])
            func_obj(user, _ip, password)

    def run(self):
        Cmd.run(f"rm -rf ~/.ssh/known_hosts {self.empty}")

        if self.group_type:
            print("┏" + "━" * 56 + "┓")
            print("┃" + "YOUQU3 REMOTE DRIVER".center(56) + "┃")
            print("┣" + "━" * 56 + "┫")
            print(
                f"┃{'GROUPS'.center(8)}┃{'CLIENTS'.center(9)}┃{'USER'.center(10)}┃{'IP'.center(15)}┃{'PASSWORD'.center(10)}┃")
            for group, clients in self.cli_groups.items():
                print("┣" + "━" * 56 + "┫")
                for c, (user, _ip, password) in clients.items():
                    print(f"┃{group.center(8)}┃{c.center(9)}┃{user.center(10)}┃{_ip.center(15)}┃{password.center(10)}┃")
            print("┗" + "━" * 56 + "┛")

            def split_case(lst, n):
                k, m = divmod(len(lst), n)
                return [lst[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in range(n)]

            task = []
            collection_cases = self.get_collection_only_cases
            executor = ThreadPoolExecutor()
            for group, clients in self.cli_groups.items():
                _client_name_list = list(clients.keys())
                client_name_list = [i for i in _client_name_list if not i.endswith("-X")]
                if len(client_name_list) > 1:
                    client_cases_map = dict(
                        zip(client_name_list, split_case(list(collection_cases), len(client_name_list)))
                    )
                    for client_name in client_name_list:
                        # self.group_client_worker(client_cases_map, client_name, clients)
                        t = executor.submit(self.group_client_worker, client_cases_map, client_name, clients)
                        task.append(t)

                else:
                    if client_name_list:
                        # self.client_worker(client_name_list, clients)
                        t = executor.submit(self.client_worker, client_name_list, clients)
                        task.append(t)

            wait(task, return_when=ALL_COMPLETED)

        else:
            print("┏" + "━" * 47 + "┓")
            print("┃" + "YOUQU3 REMOTE DRIVER".center(47) + "┃")
            print("┣" + "━" * 47 + "┫")
            print(f"┃{'CLIENTS'.center(9)}┃{'USER'.center(10)}┃{'IP'.center(15)}┃{'PASSWORD'.center(10)}┃")
            print("┣" + "━" * 47 + "┫")
            for c, (user, _ip, password) in self.cli_clients.items():
                print(f"┃{c.center(9)}┃{user.center(10)}┃{_ip.center(15)}┃{password.center(10)}┃")
            print("┗" + "━" * 47 + "┛")

            _client_name_list = list(self.cli_clients.keys())
            client_name_list = [i for i in _client_name_list if not i.endswith("-X")]
            self.mul_do(self.send_code_and_env, client_name_list, self.cli_clients)
            self.parallel_run(client_name_list, self.cli_clients)
            self.get_back_all_report(client_name_list, self.cli_clients)

    def client_worker(self, client_name_list, clients):
        client_info = clients.get(client_name_list[0])
        self.send_code_and_env(*client_info)
        self.run_test(*client_info, use_sshpass=True)
        self.get_back_report(*client_info)

    def group_client_worker(self, client_cases_map, client_name, clients):
        client_info = clients.get(client_name)
        client_cases = client_cases_map.get(client_name)
        self.inside_filepath = " ".join(client_cases)
        self.send_code_and_env(*client_info)
        self.run_test(*client_info, use_sshpass=True)
        self.get_back_report(*client_info)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import pathlib
import re
import shutil
from time import strftime

from youqu3 import setting


class Init:

    @staticmethod
    def copy_template_to_apps():
        os.system(f"cp -r {setting.TPL_PATH}/* .")
        os.system(f"cp {setting.TPL_PATH}/.gitignore-tpl  .")
        os.system(f"cp {setting.TPL_PATH}/.env-tpl  .")

    @classmethod
    def init(cls):
        cls.copy_template_to_apps()
        dirname = pathlib.Path("driver").absolute().name
        for root, dirs, files in os.walk("driver"):
            for file in files:
                app_name = dirname.lower()

                if file.endswith("-tpl"):

                    if "${app_name}" in file:
                        new_file = re.sub(
                            r"\${app_name}", app_name, file
                        )
                        shutil.move(f"{root}/{file}", f"{root}/{new_file}")
                        file = new_file

                    if ".py-tpl" in file or ".csv-tpl" in file or ".md-tpl" in file:
                        with open(f"{root}/{file}", "r") as f:
                            codes = f.readlines()
                        new_codes = []
                        for code in codes:
                            if "${APP_NAME}" in code:
                                code = re.sub(r"\${APP_NAME}", app_name.upper(), code)
                            if "${app_name}" in code:
                                code = re.sub(r"\${app_name}", app_name, code)
                            if "${AppName}" in code:
                                code = re.sub(r"\${AppName}", app_name.title().replace("_", "").replace("-", ""), code)
                            if "${USER}" in code:
                                code = re.sub(r"\${USER}", setting.USERNAME, code)
                            if "${DATE}" in code:
                                code = re.sub(r"\${DATE}", strftime("%Y/%m/%d"), code)
                            if "${TIME}" in code:
                                code = re.sub(r"\${TIME}", strftime("%H:%M:%S"), code)
                            if "${FIXEDCSVTITLE}" in code:
                                code = re.sub(
                                    r"\${FIXEDCSVTITLE}",
                                    ",".join([i.value for i in setting.FixedCsvTitle]),
                                    code,
                                )
                            new_codes.append(code)
                        with open(f"{root}/{file}", "w") as f:
                            f.writelines([i for i in new_codes])

                    shutil.move(f"{root}/{file}", f"{root}/{file[:-4]}")


class Tree:
    class Node:
        def __init__(self, name):
            self.name = name
            self.children = []

        def add_child(self, child):
            if child not in self.children:
                self.children.append(child)

        def __repr__(self, ):
            return str(self.name)

    def __init__(self, root_name, edges):
        self.edges = edges
        self.root = self._build_tree(root_name)
        self.tree_plot = ''

    def _build_tree(self, root_name):
        root = self.Node(root_name)
        for e in self.edges:
            if e[0] == root_name:
                root.add_child(self._build_tree(e[1]))
        return root

    def _tree(self, node, prefix=[]):
        space = '     '
        branch = '│   '
        tee = '├─ '
        last = '└─ '

        if len(prefix) == 0:
            self.tree_plot = ''

        self.tree_plot += ''.join(prefix) + str(node) + '\n'

        if len(prefix) > 0:
            prefix[-1] = branch if prefix[-1] == tee else space

        for i, e in enumerate(node.children):
            if i < len(node.children) - 1:
                self._tree(e, prefix + [tee])
            else:
                self._tree(e, prefix + [last])

        return self.tree_plot

    def __repr__(self):
        return self._tree(self.root)


def print_tree():
    from youqu3 import version
    print(f"The project has been init by YouQu3-{version}")
    edges = []
    for root, dirs, files in os.walk("driver"):
        dirs.sort()
        files.sort()
        if root.startswith(("./.idea", "./.vscode")):
            continue
        _dirname = os.path.split(os.path.abspath(root))[-1]
        for dir in dirs:
            if dir in (".idea", ".vscode"):
                continue
            edges.append((_dirname, dir))
        for file in files:
            if file in ("readme", "__init__.py"):
                continue
            edges.append((_dirname, file))

    dirname = pathlib.Path("driver").absolute().name
    print(Tree(dirname, edges))



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import pathlib
import re
from collections import Counter

from youqu3._setting._setting import setting


class Py2Csv:

    def __init__(self, walkdir=None):
        self.walkdir = walkdir or "."
        self.rootdir = pathlib.Path("driver").absolute()
        self.report_path = self.rootdir / "report"

    def scan_csv_and_py(self):
        csv_path_dict = {}
        py_path_dict = {}
        for root, _, files in os.walk(self.walkdir):
            py_files = []
            for file in files:
                if file.endswith(".csv"):
                    csv_path_dict[os.path.splitext(file)[0]] = f"{root}/{file}"
                if file.startswith("test_") and file.endswith(".py"):
                    case_name = []
                    _case_name = re.findall(
                        r"test_(.*?)_(\d+)_\d+.py|test_(.*?)_(\d+).py", file
                    )
                    if _case_name:
                        _case_name = _case_name[0]
                        if isinstance(_case_name, tuple):
                            for i in _case_name:
                                if i:
                                    case_name.append(i)

                    if py_path_dict.get(case_name[0]):
                        tmp_py_list = py_path_dict.get(case_name[0])
                        tmp_py_list.append([f"{root}/{file}", root, case_name[-1]])
                        py_path_dict[case_name[0]] = tmp_py_list
                    else:
                        py_files.append([f"{root}/{file}", root, case_name[-1]])
                        py_path_dict[case_name[0]] = py_files

        for i in py_path_dict:
            py_path_dict[i] = sorted(py_path_dict[i], key=lambda x: int(x[-1]))
        return csv_path_dict, py_path_dict

    def delete_mark_in_csv_if_not_exists_py(self):
        res = self.scan_csv_and_py()
        if res is None:
            return
        csv_path_dict, py_path_dict = res
        flag = False
        for csv_name in csv_path_dict:
            for case_name in py_path_dict:
                if csv_name == case_name:
                    csv_path = csv_path_dict.get(csv_name)
                    with open(csv_path, "r", encoding="utf-8") as f:
                        csv_txt_list = f.readlines()
                    taglines = [txt.strip().split(",") for txt in csv_txt_list[1:]]
                    new_taglines = []
                    py_case_paths = py_path_dict.get(case_name)
                    for tag in taglines:
                        try:
                            csv_case_id = f"{int(tag[0]):0>3}"
                        except ValueError as e:
                            raise ValueError(f"文件：{csv_path} 里面似乎格式有点问题,出现了一个报错：{e}")
                        for py_case in py_case_paths:
                            py_case_id = py_case[-1]
                            if csv_case_id == py_case_id:
                                new_taglines.append(tag)
                                break
                        else:
                            print(f"{tag} will remove from {csv_path}")
                    if new_taglines != taglines:
                        bak_path = f"{self.report_path}/pyid2csv_back"
                        if not os.path.exists(bak_path):
                            os.makedirs(bak_path)
                        os.system(
                            f"cp {csv_path} {bak_path}/{setting.TIME_STRING}_{csv_name}.csv"
                        )
                        new_csv_list = [csv_txt_list[0]] + [
                            ",".join(i) + "\n" for i in new_taglines
                        ]
                        with open(csv_path, "w+", encoding="utf-8") as f:
                            f.writelines(new_csv_list)
                        flag = True
        if flag:
            print("The corresponding script ID no longer exists in the CSV file after deletion")

    def async_mark_to_csv(self):
        self.csv_path_dict, self.py_path_dict = self.scan_csv_and_py()
        for case_name in self.py_path_dict:
            py_paths = self.py_path_dict.get(case_name)
            py_count = Counter([i[1] for i in py_paths])
            is_one = len(py_paths) == 1 or len(py_count) == 1
            for py_path, py_dirpath, case_id in py_paths:
                if not self.csv_path_dict or not self.csv_path_dict.get(case_name):
                    _dir_name = os.path.dirname(os.path.abspath(py_path))
                    if is_one:
                        if str(_dir_name).endswith("case"):
                            dir_name = self.walkdir
                        else:
                            dir_name = os.path.dirname(_dir_name.replace("/case/", "/tag/"))
                    else:
                        dir_name = self.walkdir
                    if not os.path.exists(dir_name):
                        os.makedirs(dir_name)

                    csv_path = f"{dir_name}/{case_name}.csv"
                    with open(csv_path, "w+", encoding="utf-8") as f:
                        f.write(",".join([i.value for i in setting.FixedCsvTitle]) + "\n")

                    self.csv_path_dict, self.py_path_dict = self.scan_csv_and_py()

                csv_path = self.csv_path_dict.get(case_name)
                with open(csv_path, "r", encoding="utf-8") as f:
                    csv_txt_list = f.readlines()
                    try:
                        csv_head = csv_txt_list[0]
                        comma_num = csv_head.count(",")
                    except IndexError:
                        with open(csv_path, "w+", encoding="utf-8") as f:
                            f.write(",".join([i.value for i in setting.FixedCsvTitle]) + "\n")
                        comma_num = len(setting.FixedCsvTitle) - 1
                csv_taglines = [txt.strip().split(",") for txt in csv_txt_list[1:]]
                if not csv_taglines:
                    with open(csv_path, "a+", encoding="utf-8") as f:
                        f.write(f"{case_id}{comma_num * ','}" + "\n")
                else:
                    for i in csv_taglines:
                        if i[0] == case_id or int(i[0]) == int(case_id):
                            break
                    else:
                        with open(csv_path, "a+", encoding="utf-8") as f:
                            f.write(f"{case_id}{comma_num * ','}" + "\n")
        print("Complete synchronization of automated test script id to csv file")


if __name__ == "__main__":
    Py2Csv().delete_mark_in_csv_if_not_exists_py()
    Py2Csv().async_mark_to_csv()



import getpass
import os
import pathlib
import platform
import time
from configparser import RawConfigParser


class GetCfg:
    """Gets the value in the configuration file"""

    def __init__(self, config_file: str, option: [str, None] = None):
        self.config_file = config_file
        self.option = option
        self.conf = RawConfigParser()
        self.conf.read(self.config_file, encoding="utf-8")

    def get(self, key: str, op: [str, None] = None, default=None) -> str:
        if op is None and self.option is not None:
            op = self.option
        if op is None and self.option is None:
            raise ValueError("option is None")
        return self.conf.get(op, key, fallback=default)

    def get_bool(self, key: str, op: [str, None] = None, default=False) -> bool:
        if op is None and self.option is not None:
            op = self.option
        if op is None and self.option is None:
            raise ValueError("option is None")
        return self.conf.getboolean(op, key, fallback=default)


class ArchName:
    x86 = "x86_64"
    aarch64 = "aarch64"
    loongarch64 = "loongarch64"
    mips64 = "mips64"
    sw64 = "sw64"


class DisplayServer:
    wayland = "wayland"
    x11 = "x11"


class _DynamicSetting:
    SYS_ARCH = platform.machine().lower()
    HOME = str(pathlib.Path.home())
    USERNAME = getpass.getuser()

    YOUQU_HOME = pathlib.Path(__file__).parent.parent
    TPL_PATH = YOUQU_HOME / "tpl"

    if os.path.exists(os.path.expanduser("~/.xsession-errors")):
        DISPLAY_SERVER = (
            os.popen("cat ~/.xsession-errors | grep XDG_SESSION_TYPE | head -n 1")
            .read()
            .split("=")[-1]
            .strip("\n")
        )
    else:
        DISPLAY_SERVER = "x11" if os.popen("ps -ef | grep -v grep | grep kwin_x11").read() else "wayland"

    IS_X11: bool = DISPLAY_SERVER == DisplayServer.x11
    IS_WAYLAND: bool = DISPLAY_SERVER == DisplayServer.wayland

    TIME_STRING = time.strftime("%Y%m%d%H%M%S")
    HOST_IP = str(os.popen("hostname -I |awk '{print $1}'").read()).strip("\n").strip()

    VERSION = ""
    if os.path.exists("/etc/os-version"):
        version_cfg = GetCfg("/etc/os-version", "Version")
        VERSION = (version_cfg.get("EditionName[zh_CN]") or "") + (
                version_cfg.get("MinorVersion") or ""
        )
